#!/bin/bash

XENSTORE_READ=/usr/bin/xenstore-read
XENSTORE_WRITE=/usr/bin/xenstore-write
PYTHON=/usr/bin/python
VNSTAT=/usr/bin/vnstat
SLEEP=/bin/sleep

DEFAULT_CONFIG=/etc/default/xen

. ${DEFAULT_CONFIG}
. /etc/xen/scripts/xen-script-common.sh

ACTION_TYPE=$1
TYPEIF=$2

evalVariables "$@"

dev=vif

case "$type_if" in
    vif)
        dev=${vif}
        ;;
    tap)
        dev=$INTERFACE
        ;;
    *)
        exit 1
        ;;
esac

if [ "${ACTION_TYPE}" = "online" ]
then
    if [ "${type_if}" = "vif" ]
    then
	KEY_VIFNAME=/local/domain/0/${XENBUS_PATH}/vifname
	
	if [ "${TOOLSTACK}" = "xl" ]
	then
	  dev_=${dev#vif}
	  DOMID=${dev_%.*}
	  DOMNAME_KEY=/local/domain/${DOMID}/name
	  DOMNAME=`${XENSTORE_READ} ${DOMNAME_KEY} 2>/dev/null`
	  DOM_CONFIG=/etc/xen/${DOMNAME}.cfg
	  PYTHON_CODE=`/bin/echo '#!/usr/bin/python' ; /bin/echo '' ; /bin/cat ${DOM_CONFIG} ; /bin/echo '' ; /bin/echo 'for data_vif_in in vif:' ; /bin/echo '    print data_vif_in'`
	  VARIABLES=`${PYTHON} -c "${PYTHON_CODE}" | /bin/grep vifname | /bin/sed 's/,/ ; /'`
	  eval ${VARIABLES}
	  ${XENSTORE_WRITE} ${KEY_VIFNAME} ${vifname} 1>/dev/null 2>/dev/null
	else
	  vifname=`${XENSTORE_READ} ${KEY_VIFNAME} 2>/dev/null`
	fi
	
	/etc/xen/scripts/vif-bridge $@
	${SLEEP} 9
	${VNSTAT} -u -i ${vifname} 1>/dev/null 2>/dev/null
	${VNSTAT} --enable -i ${vifname} 1>/dev/null 2>/dev/null
	${VNSTAT} --sync -i ${vifname} 1>/dev/null 2>/dev/null
	${VNSTAT} --reset -i ${vifname} 1>/dev/null 2>/dev/null
    else
	/etc/xen/scripts/vif-bridge $@
    fi
fi

if [ "${ACTION_TYPE}" = "offline" ]
then
  KEY_VIFNAME=/local/domain/0/${XENBUS_PATH}/vifname
  vifname=`${XENSTORE_READ} ${KEY_VIFNAME} 2>/dev/null`
  ${VNSTAT} -r --disable -i ${vifname} 1>/dev/null 2>/dev/null
  /etc/xen/scripts/vif-bridge $@
fi

if [ "${ACTION_TYPE}" = "add" ]
then
  /etc/xen/scripts/vif-bridge $@
fi






