#!/bin/bash

XENSTORE_READ=/usr/bin/xenstore-read
VNSTAT=/usr/bin/vnstat
IP=/sbin/ip
OVS_VSCTL=/usr/bin/ovs-vsctl
VIF_SCRIPT=/etc/xen/scripts/vif-bridge
XEN_SCRIPT_COMMON=/etc/xen/scripts/xen-script-common.sh

THIS_FILE=`basename $0`

FILE_LOG_ERRORS=/var/log/${THIS_FILE}.log

. ${XEN_SCRIPT_COMMON}

ACTION_TYPE=$1
TYPEIF=$2

evalVariables "$@"

dev=vif

case "$type_if" in
    vif)
        dev=${vif}
        ;;
    tap)
        dev=$INTERFACE
        ;;
    *)
        exit 1
        ;;
esac

if [ "${ACTION_TYPE}" = "online" ] || [ "${ACTION_TYPE}" = "add" ]
then
    if [ "${type_if}" = "vif" ]
    then
        ${VIF_SCRIPT} $@
        KEY_VIFNAME=/local/domain/0/${XENBUS_PATH}/vifname
        vifname=`${XENSTORE_READ} ${KEY_VIFNAME} 2>/dev/null`
        ${VNSTAT} -u -i ${vifname} 1>/dev/null 2>>${FILE_LOG_ERRORS}
        ${VNSTAT} --enable -i ${vifname} 1>/dev/null 2>>${FILE_LOG_ERRORS}
        ${VNSTAT} --sync -i ${vifname} 1>/dev/null 2>>${FILE_LOG_ERRORS}
        ${VNSTAT} --reset -i ${vifname} 1>/dev/null 2>>${FILE_LOG_ERRORS}
    else
        ${VIF_SCRIPT} $@
    fi
fi


if [ "${ACTION_TYPE}" = "offline" ] || [ "${ACTION_TYPE}" = "remove" ]
then
    if [ "${type_if}" = "vif" ]
    then
        KEY_VIFNAME=/local/domain/0/${XENBUS_PATH}/vifname
        vifname=`${XENSTORE_READ} ${KEY_VIFNAME} 2>/dev/null`
        ${VNSTAT} -r --disable -i ${vifname} 1>/dev/null 2>>${FILE_LOG_ERRORS}
        ${VIF_SCRIPT} $@
    else
        ${VIF_SCRIPT} $@
    fi
fi








