#!/bin/bash

XENSTORE_READ=/usr/bin/xenstore-read
XENSTORE_WRITE=/usr/bin/xenstore-write
XENSTORE_LIST=/usr/bin/xenstore-list
VNSTAT=/usr/bin/vnstat
IP=/sbin/ip
OVS_VSCTL=/usr/bin/ovs-vsctl
VIF_SCRIPT=/etc/xen/scripts/vif-openvswitch
XEN_SCRIPT_COMMON=/etc/xen/scripts/xen-script-common.sh
ECHO=/bin/echo
MAWK=/usr/bin/mawk

set_vifname_on()
{
    vifname_on=$1
    VIF_ORIG_on=$2
    vm_mac_on=$3
    VM_UUID_on=$4
    VM_NAME_on=$5
    vm_bridge_on=$6

    MAC_PREF="FE"

    if [ -n "${vm_mac_on}" ]
    then
	vm_mac_dom0=${vm_mac_on}
	vm_mac_dom0=${MAC_PREF}${vm_mac_dom0:2}
    fi

    if ${IP} link show dev ${VIF_ORIG_on} 1>/dev/null 2>/dev/null
    then
	if ${IP} link show dev ${vifname_on} 1>/dev/null 2>/dev/null
	then
	    ${IP} link set dev ${vifname_on} down 1>/dev/null 2>>${FILE_LOG_ERRORS} || true
	    ${IP} link delete dev ${vifname_on} 1>/dev/null 2>>${FILE_LOG_ERRORS} || true
	fi

	${IP} link set dev ${VIF_ORIG_on} down 1>/dev/null 2>>${FILE_LOG_ERRORS} || true
	${IP} link set ${VIF_ORIG_on} name ${vifname_on} 1>/dev/null 2>>${FILE_LOG_ERRORS} || true
	${IP} link set ${vifname_on} address ${vm_mac_dom0} 1>/dev/null 2>>${FILE_LOG_ERRORS} || true
	${IP} address flush dev ${vifname_on} 1>/dev/null 2>>${FILE_LOG_ERRORS} || true
    fi

    if ${IP} link show dev ${vifname_on} 1>/dev/null 2>/dev/null
    then
	bridge_p_on=${vm_bridge_on}
	tag_p_on=""
	trunk_p_on=""
	trunk_arg_on=""
	tag_arg_on=""

	${IP} link set dev ${vifname_on} up 1>/dev/null 2>>${FILE_LOG_ERRORS} || true
	${IP} link set dev ${vifname_on} promisc on 1>/dev/null 2>>${FILE_LOG_ERRORS} || true

	if [[ ${bridge_p_on} =~ ^([^.:]+)(\.([[:digit:]]+))?(:([[:digit:]]+(:[[:digit:]]+)*))?$ ]]
	then
	    bridge_p_on="${BASH_REMATCH[1]}"
	    tag_p_on="${BASH_REMATCH[3]}"
	    trunk_p_on="${BASH_REMATCH[5]//:/,}"
	else
	    ${ECHO} "No valid bridge was specified"
	    exit 1
	fi

	if [ ${trunk_p_on} ]
	then
	    trunk_arg_on="trunk=${trunk_p_on}"
	fi

	if [ ${tag_p_on} ]
	then
	    tag_arg_on="tag=${tag_p_on}"
	fi

	${OVS_VSCTL} -- --if-exists del-port ${bridge_p_on} ${vifname_on} 1>/dev/null 2>>${FILE_LOG_ERRORS}
	${OVS_VSCTL} -- --may-exist add-port ${bridge_p_on} ${vifname_on} ${tag_arg_on} ${trunk_arg_on} 1>/dev/null 2>>${FILE_LOG_ERRORS}
	${OVS_VSCTL} set interface ${vifname_on} external-ids:"attached-mac"="${vm_mac_on}" 1>/dev/null 2>>${FILE_LOG_ERRORS}
	${OVS_VSCTL} set interface ${vifname_on} external-ids:"xen-vm-uuid"="${VM_UUID_on}" 1>/dev/null 2>>${FILE_LOG_ERRORS}
	${OVS_VSCTL} set interface ${vifname_on} external-ids:"xen-vm-name"="${VM_NAME_on}" 1>/dev/null 2>>${FILE_LOG_ERRORS}

	${XENSTORE_WRITE} ${KEY_HOTPLUG} connected 1>/dev/null 2>>${FILE_LOG_ERRORS}

	${VNSTAT} -u -i ${vifname_on} 1>/dev/null 2>>${FILE_LOG_ERRORS}
	${VNSTAT} --enable -i ${vifname_on} 1>/dev/null 2>>${FILE_LOG_ERRORS}
	${VNSTAT} --sync -i ${vifname_on} 1>/dev/null 2>>${FILE_LOG_ERRORS}
	${VNSTAT} --reset -i ${vifname_on} 1>/dev/null 2>>${FILE_LOG_ERRORS}
    fi
}

set_vifname_off()
{
    vifname_off=$1
    vm_bridge_off=$2

    ${VNSTAT} -r --disable -i ${vifname_off} 1>/dev/null 2>>${FILE_LOG_ERRORS}
    ${OVS_VSCTL} -- --if-exists del-port ${vm_bridge_off} ${vifname_off} 1>/dev/null 2>>${FILE_LOG_ERRORS}

    if ${IP} link show dev ${vifname_off} 1>/dev/null 2>/dev/null
    then
	${IP} link set dev ${vifname_off} down 1>/dev/null 2>>${FILE_LOG_ERRORS} || true
	${IP} link delete dev ${vifname_off} 1>/dev/null 2>>${FILE_LOG_ERRORS} || true
    fi
}

set_rename_vif()
{
    vifname_ren=$1
    VIF_ORIG_ren=$2
    vm_mac_ren=$3
    VM_UUID_ren=$4
    VM_NAME_ren=$5
    vm_bridge_ren=$6
    virt_type_ren=$7

    MAC_PREF="FE"

    vm_mac_dom0=""

    VIF_ORIG_VIF_ren=${VIF_ORIG_ren}

    if [ "${virt_type}" = "hvm" ]
    then
	VIF_ORIG_ren=${VIF_ORIG_ren}-emu

	if ${IP} link show "${VIF_ORIG_VIF_ren}" >&/dev/null
	then
	    ${IP} link set dev ${VIF_ORIG_VIF_ren} down 1>/dev/null 2>>${FILE_LOG_ERRORS} || true
	    ##${IP} link delete dev ${VIF_ORIG_VIF_ren} 1>/dev/null 2>>${FILE_LOG_ERRORS} || true
	fi
    fi

    if [ -n "${vm_mac_ren}" ]
    then
	vm_mac_dom0=${vm_mac_ren}
	vm_mac_dom0=${MAC_PREF}${vm_mac_dom0:2}
    fi

    if ${IP} link show "$vifname_ren" >&/dev/null
    then
	${IP} link set dev ${vifname_ren} down 1>/dev/null 2>>${FILE_LOG_ERRORS} || true
	${IP} link delete dev ${vifname_ren} 1>/dev/null 2>>${FILE_LOG_ERRORS} || true

	${IP} link set dev ${VIF_ORIG_ren} down 1>/dev/null 2>>${FILE_LOG_ERRORS} || true
	${IP} link set "${VIF_ORIG_ren}" name "${vifname_ren}"
    else
	${IP} link set dev ${VIF_ORIG_ren} down 1>/dev/null 2>>${FILE_LOG_ERRORS} || true
	${IP} link set "${VIF_ORIG_ren}" name "${vifname_ren}"
    fi

    bridge_p_ren=${vm_bridge_ren}
    tag_p_ren=""
    trunk_p_ren=""
    trunk_arg_ren=""
    tag_arg_ren=""

    ${IP} link set ${vifname_ren} address ${vm_mac_dom0} 1>/dev/null 2>>${FILE_LOG_ERRORS} || true
    ${IP} link set dev ${vifname_ren} up 1>/dev/null 2>>${FILE_LOG_ERRORS} || true
    ${IP} link set dev ${vifname_ren} promisc on 1>/dev/null 2>>${FILE_LOG_ERRORS} || true
    ${IP} address flush dev ${vifname_ren} 1>/dev/null 2>>${FILE_LOG_ERRORS} || true

    if [[ ${bridge_p_ren} =~ ^([^.:]+)(\.([[:digit:]]+))?(:([[:digit:]]+(:[[:digit:]]+)*))?$ ]]
    then
	bridge_p_ren="${BASH_REMATCH[1]}"
	tag_p_ren="${BASH_REMATCH[3]}"
	trunk_p_ren="${BASH_REMATCH[5]//:/,}"
    else
	${ECHO} "No valid bridge was specified"
	exit 1
    fi

    if [ ${trunk_p_ren} ]
    then
	trunk_arg_ren="trunk=${trunk_p_ren}"
    fi

    if [ ${tag_p_ren} ]
    then
	tag_arg_on="tag=${tag_p_ren}"
    fi

    ${OVS_VSCTL} -- --if-exists del-port ${bridge_p_ren} ${vifname_ren} 1>/dev/null 2>>${FILE_LOG_ERRORS}
    ${OVS_VSCTL} -- --may-exist add-port ${bridge_p_ren} ${vifname_ren} ${tag_arg_ren} ${trunk_arg_ren} 1>/dev/null 2>>${FILE_LOG_ERRORS}
    ${OVS_VSCTL} set interface ${vifname} external-ids:"attached-mac"="${vm_mac_ren}" 1>/dev/null 2>>${FILE_LOG_ERRORS}
    ${OVS_VSCTL} set interface ${vifname} external-ids:"xen-vm-uuid"="${VM_UUID_ren}" 1>/dev/null 2>>${FILE_LOG_ERRORS}
    ${OVS_VSCTL} set interface ${vifname} external-ids:"xen-vm-name"="${VM_NAME_ren}" 1>/dev/null 2>>${FILE_LOG_ERRORS}

    ${XENSTORE_WRITE} ${KEY_HOTPLUG} connected 1>/dev/null 2>>${FILE_LOG_ERRORS}

    ${VNSTAT} -u -i ${vifname_ren} 1>/dev/null 2>>${FILE_LOG_ERRORS}
    ${VNSTAT} --enable -i ${vifname_ren} 1>/dev/null 2>>${FILE_LOG_ERRORS}
    ${VNSTAT} --sync -i ${vifname_ren} 1>/dev/null 2>>${FILE_LOG_ERRORS}
    ${VNSTAT} --reset -i ${vifname_ren} 1>/dev/null 2>>${FILE_LOG_ERRORS}
}

THIS_FILE=`basename $0`

FILE_LOG_ERRORS=/var/log/${THIS_FILE}.log

. ${XEN_SCRIPT_COMMON}

ACTION_TYPE=$1
TYPEIF=$2

evalVariables "$@"

dev=vif

case "$type_if" in
    vif)
        dev=${vif}
        ;;
    tap)
        dev=$INTERFACE
        ;;
    *)
        exit 1
        ;;
esac

if [ "${ACTION_TYPE}" = "online" ] || [ "${ACTION_TYPE}" = "add" ]
then
    if [ "${type_if}" = "vif" ] || [ "${type_if}" = "tap" ]
    then
        KEY_BRIDGE=/local/domain/0/${XENBUS_PATH}/bridge
        vm_bridge=`${XENSTORE_READ} ${KEY_BRIDGE} 2>/dev/null`
        KEY_VIFNAME=/local/domain/0/${XENBUS_PATH}/vifname
        vifname=`${XENSTORE_READ} ${KEY_VIFNAME} 2>/dev/null`

        domid_=${vif#vif}
        domid_=${domid_%-emu}
        domid=${domid_%.*}

	vifname_hvm=${vifname%-emu}

        KEY_VIRT_TYPE=/libxl/${domid}/type
        virt_type=`${XENSTORE_READ} ${KEY_VIRT_TYPE} 2>/dev/null`

        MAC_PATH=/local/domain/0/${XENBUS_PATH}/mac
        vm_mac=`${XENSTORE_READ} ${MAC_PATH} 2>/dev/null`
        FRONTEND_ID_PATH=/local/domain/0/${XENBUS_PATH}/frontend-id

        KEY_HOTPLUG=/local/domain/0/${XENBUS_PATH}/hotplug-status

        frontend_id=`${XENSTORE_READ} ${FRONTEND_ID_PATH} 2>/dev/null`
        VM_PATH=`${XENSTORE_READ} /local/domain/${frontend_id}/vm`
        VM_NAME=`${XENSTORE_READ} ${VM_PATH}/name`
        VM_UUID=`${XENSTORE_READ} ${VM_PATH}/uuid`

        VIF_ORIG=${vif}

	if [ "${virt_type}" = "hvm" ]
	then
	    if [ ${type_if} = "tap"  ]
	    then
		set_rename_vif ${vifname} ${VIF_ORIG} ${vm_mac} ${VM_UUID} ${VM_NAME} ${vm_bridge} ${virt_type}
	    fi
	else
	    set_vifname_on ${vifname} ${VIF_ORIG} ${vm_mac} ${VM_UUID} ${VM_NAME} ${vm_bridge}
	fi
    fi
fi

if [ "${ACTION_TYPE}" = "offline" ] || [ "${ACTION_TYPE}" = "remove" ]
then
    if [ "${type_if}" = "vif" ] || [ "${type_if}" = "tap" ]
    then
        KEY_BRIDGE=/local/domain/0/${XENBUS_PATH}/bridge
        vm_bridge=`${XENSTORE_READ} ${KEY_BRIDGE} 2>/dev/null`
        KEY_VIFNAME=/local/domain/0/${XENBUS_PATH}/vifname
        vifname=`${XENSTORE_READ} ${KEY_VIFNAME} 2>/dev/null`
	vifname_hvm=${vifname%-emu}

        domid_=${vif#vif}
        domid_=${domid_%-emu}
        domid=${domid_%.*}

        KEY_VIRT_TYPE=/libxl/${domid}/type
        virt_type=`${XENSTORE_READ} ${KEY_VIRT_TYPE} 2>/dev/null`

	if [ "${virt_type}" = "hvm" ]
	then
	    if [ ${type_if} = "tap"  ]
	    then
		set_vifname_off ${vifname} ${vm_bridge}
	    else
		set_vifname_off ${vifname} ${vm_bridge}
	    fi
	else
	    set_vifname_off ${vifname} ${vm_bridge}
	fi
    fi
fi








